// here normalization is used in the comparison

package jmetal.util.comparators;

import jmetal.core.Solution;

import java.util.Comparator;

/**
 * This class implements a <code>Comparator</code> (a method for comparing
 * <code>Solution</code> objects) based on a constraint violation test + 
 * dominance checking, as in NSGA-II.
 */
public class AngleComparator3 implements Comparator{
 
  /** 
   * stores a comparator for check the OverallConstraintComparator
   */
  private static final Comparator overallConstraintViolationComparator_ =
                              new OverallConstraintViolationComparator();
 /**
  * Compares two solutions.
  * @param object1 Object representing the first <code>Solution</code>.
  * @param object2 Object representing the second <code>Solution</code>.
  * @return -1, or 0, or 1 if solution1 dominates solution2, both are 
  * non-dominated, or solution1  is dominated by solution22, respectively.
  */
  public int compare(Object object1, Object object2) {
    if (object1==null)
      return 1;
    else if (object2 == null)
      return -1;
    
    Solution solution1 = (Solution)object1;
    Solution solution2 = (Solution)object2;

    int dominate1 ; // dominate1 indicates if some objective of solution1 
                    // dominates the same objective in solution2. dominate2
    int dominate2 ; // is the complementary of dominate1.

    dominate1 = 0 ; 
    dominate2 = 0 ;
    
    int flag; //stores the result of the comparison

    if (solution1.getOverallConstraintViolation()!= 
        solution2.getOverallConstraintViolation() &&
       (solution1.getOverallConstraintViolation() < 0) ||         
       (solution2.getOverallConstraintViolation() < 0)){            
      return (overallConstraintViolationComparator_.compare(solution1,solution2));
    }
                                                
    // Equal number of violated constraints. Applying a dominance Test then
    double value1, value2;
    for (int i = 0; i < solution1.numberOfObjectives(); i++) {
      value1 = solution1.getObjective(i);
      value2 = solution2.getObjective(i);
      if (value1 < value2) {
        flag = -1;
      } else if (value1 > value2) {
        flag = 1;
      } else {
        flag = 0;
      }
      
      if (flag == -1) {
        dominate1 = 1;
      }
      
      if (flag == 1) {
        dominate2 = 1;           
      }
    }
            
    if (dominate1 == dominate2) {            
      return 0; //No one dominate the other
    }
    if (dominate1 == 1) {
      return -1; // solution1 dominate
    }
    return 1;    // solution2 dominate   
  } // compare
  
  /**
   * Extreme_DOminance_Relationship
   * @param A
   * @param B
   * @param extreme[0]: ideal point 
   * @param extreme[1]: nadir point 
   * @return
   */
  public int compare(Solution A, Solution B, double[][] extreme)  {

	  int nobj = A.numberOfObjectives();
	  double[] s1 = new double[nobj];
	  double[] s2 = new double[nobj];
	  
	  // normalization
	  for(int i=0;i<nobj;i++) {
		  s1[i] = (A.getObjective(i)-extreme[0][i])/(extreme[1][i]-extreme[0][i]);
		  s2[i] = (B.getObjective(i)-extreme[0][i])/(extreme[1][i]-extreme[0][i]);
	  }
	  
	  // angle-dominance
	  double value1, value2;
	  double mid = Math.PI/4;
	  double theta2 = Math.PI/3;
//	  double theta2 = Math.PI/4;
//	  double theta2 = 5*Math.PI/12;
//	  double theta2 = Math.PI/2;
	  
	  /***************************************************************/
	  
	// do judgment 
	    double[] standard1 = new double[nobj];
	    standard1[0] = 1;
	    for(int i=1;i<nobj;i++) {
	    	standard1[i] = 0;
	    }
	    
	       
	    double alpha1 = calcAngle(s1, standard1);
	    double alpha2 = calcAngle(s2, standard1);
	    
	    // AB
	    double[] standard2 = new double[nobj];
	    double[] AB = new double[nobj];
	    // BA
	    double[] BA = new double[nobj];  
	    for (int i=0;i<nobj;i++) {
	    	AB[i] = s2[i] - s1[i];
	    	standard2[i] = 1;
	    	BA[i] = -AB[i];
	    }
	    
	    /*------------------ classification -------------------------*/
	    // -1 : solution1 better; 1 solution2 better; 0 equal.
	    if ((alpha1>mid && alpha2<mid)||(alpha1<mid && alpha2>mid)) {
	    	return 0; 
		}else if (alpha1>mid && alpha2>mid) {
			if(calcAngle(AB, s1)<theta2) { // solution 1 dominates
		    	return -1;
		    }else if (calcAngle(BA, s2)<theta2) { // solution 2 dominates
				return 1;
			}else {
				return 0; 
			}
		}else { // if (alpha1<mid && alpha2<mid)
			if(calcAngle(AB, s1)<theta2) { // solution 1 dominates
		    	return -1;
		    }else if (calcAngle(BA, s2)<theta2) { // solution 2 dominates
				return 1;
			}else {
				return 0; 
			}
		}
}
  
  /**
	 * Calc the angel between two vectors
	 * @param A
	 * @param standard
	 * @return
	 */
	public double calcAngle(double[] A, double[] standard){
		int size = A.length;
		double lenA = 0.0, lenSta = 0.0;
		double theta = 0.0;
		
		for(int i=0;i<size;i++) {
			lenA += A[i]*A[i];
			lenSta += standard[i]*standard[i];
		}
		lenA = Math.sqrt(lenA);
		lenSta = Math.sqrt(lenSta);
		
		double tmp = 0.0;
		for(int j=0;j<size;j++) {
			tmp += A[j]*standard[j];
		}
		
		theta = Math.acos(tmp/(lenA*lenSta));
		
		return theta;	
	}
  
} // DominanceComparator
